@@ -214,8 +214,9 @@ MriImage <- setRefClass("MriImage", contains="SerialisableObject", fields=list(i
         }
         else
         {
-            # This "apply" call is a cheeky bit of R wizardry (credit: Peter Dalgaard)
-            newData <- base::apply(data, dimsToKeep, "[", loc)
+            indices <- alist(x=,y=,z=,t=,u=,v=,w=)[seq_along(imageDims)]
+            indices[dim] <- loc
+            newData <- do.call("[", c(list(data),indices))
             if (is.vector(newData))
                 newData <- promote(newData)
         }
