@@ -1,4 +1,39 @@
-DicomMetadata <- setRefClass("DicomMetadata", contains="SerialisableObject", fields=list(source="character",tags="data.frame",tagOffset="integer",dataOffset="integer",dataLength="integer",explicitTypes="logical",endian="character"), methods=list(
+#' The DicomMetadata class
+#' 
+#' This class represents DICOM metadata, which typically contains detailed
+#' information about the scan parameters and subject.
+#' 
+#' @field source String naming the source file
+#' @field tags Data frame of tag information
+#' @field tagOffset Starting offset for tags in the file
+#' @field dataOffset Starting offset for pixel data in the file
+#' @field dataLength Pixel data length
+#' @field explicitTypes Logical value indicating whether explicit types are
+#'   used in the file
+#' @field endian String naming the endianness of the file
+#' @field asciiFields Character vector containing the contents of the ASCII
+#'   header, if requested and present in the file.
+#' 
+#' @export
+DicomMetadata <- setRefClass("DicomMetadata", contains="SerialisableObject", fields=list(source="character",tags="data.frame",tagOffset="integer",dataOffset="integer",dataLength="integer",explicitTypes="logical",endian="character",asciiFields="character"), methods=list(
+    getAsciiFields = function (regex = NULL)
+    {
+        "Retrieve the value of one or more fields in the ASCII header. Returns NA if no fields match"
+        if (is.null(regex))
+            return (asciiFields)
+        
+        regex <- ore("^\\s*(\\S*", regex, "\\S*)\\s*=\\s*(.+)\\s*$")
+        groups <- groups(ore.search(regex, asciiFields, simplify=FALSE), simplify=TRUE)
+        if (all(is.na(groups)))
+            return (NA)
+        
+        values <- groups[,ncol(groups),]
+        if (all(values %~% ore(number)))
+            values <- as.numeric(values)
+        names(values) <- groups[,ncol(groups)-1,]
+        return (values)
+    },
+    
     getDataLength = function () { return (dataLength) },
     
     getDataOffset = function () { return (dataOffset) },
@@ -13,6 +48,7 @@ DicomMetadata <- setRefClass("DicomMetadata", contains="SerialisableObject", fie
     
     getTagValue = function (group, element)
     {
+        "Retrieve the value of a given tag, using an appropriate R type. Returns NA if the tag is missing"
         valueRow <- subset(tags, (tags$groups == group & tags$elements == element))
         if (dim(valueRow)[1] == 0 || valueRow$values == "")
             return (NA)
@@ -33,16 +69,11 @@ DicomMetadata <- setRefClass("DicomMetadata", contains="SerialisableObject", fie
     nTags = function () { return (nrow(tags)) }
 ))
 
+#' @export
 print.DicomMetadata <- function (x, descriptions = FALSE, ...)
 {
     tags <- x$getTags()
     nTags <- nrow(tags)
-    if (descriptions && !exists("dictionary"))
-    {
-        # First set to NULL to keep package checker happy
-        dictionary <- NULL
-        data("dictionary", package="tractor.base", envir=environment(NULL))
-    }
     
     if (nTags > 0)
     {
@@ -51,7 +82,7 @@ print.DicomMetadata <- function (x, descriptions = FALSE, ...)
             cat("DESCRIPTION", rep(" ",19), "VALUE\n", sep="")
             for (i in seq_len(nTags))
             {
-                description <- getDescriptionForDicomTag(tags$groups[i], tags$elements[i], dictionary)
+                description <- getDescriptionForDicomTag(tags$groups[i], tags$elements[i])
                 cat(" ", substr(description, 1, 27), sep="")
                 nSpaces <- max(3, 30-nchar(description))
                 cat(rep(" ",nSpaces), sep="")
@@ -72,11 +103,8 @@ print.DicomMetadata <- function (x, descriptions = FALSE, ...)
     }
 }
 
-getDescriptionForDicomTag <- function (groupRequired, elementRequired, dictionary = NULL)
+getDescriptionForDicomTag <- function (groupRequired, elementRequired)
 {
-    if (is.null(dictionary))
-        data("dictionary", package="tractor.base", envir=environment(NULL))
-    
     dictionaryRow <- subset(dictionary, (dictionary$group==groupRequired & dictionary$element==elementRequired))
     if (nrow(dictionaryRow) == 0)
         description <- sprintf("Unknown (0x%04x, 0x%04x)", groupRequired, elementRequired)
@@ -86,7 +114,43 @@ getDescriptionForDicomTag <- function (groupRequired, elementRequired, dictionar
     return (description)
 }
 
-newDicomMetadataFromFile <- function (fileName, checkFormat = TRUE, dictionary = NULL, stopTag = NULL, ignoreTransferSyntax = FALSE)
+#' Read a DICOM file into a DicomMetadata object
+#' 
+#' This function reads a DICOM file into a \code{\link{DicomMetadata}} object.
+#' Only DICOM files from magnetic resonance scanners are supported.
+#' 
+#' @param fileName The name of a DICOM file.
+#' @param checkFormat If \code{TRUE}, the function will check for the magic
+#'   string \code{"DICM"} at byte offset 128. This string should be present,
+#'   but in reality not all files contain it.
+#' @param stopTag An integer vector giving the group and element numbers (in
+#'   that order) of a DICOM tag, or \code{NULL}. If not \code{NULL}, the
+#'   function will stop parsing the DICOM file if the specified tag is
+#'   encountered. This can be used to speed up the process if a specific tag is
+#'   required.
+#' @param ignoreTransferSyntax If \code{TRUE}, any transfer syntax stored in
+#'   the file will be ignored, and the code will try to deduce the transfer
+#'   syntax using heuristics. This may occasionally be necessary for awkward
+#'   DICOM files, but is not generally recommended.
+#' @param ascii If \code{TRUE}, the function will attempt to read an embedded
+#'   Siemens ASCII header, if one exists.
+#' @return \code{readDicomFile} returns a \code{\linkS4class{DicomMetadata}}
+#'   object, or \code{NULL} on failure.
+#' 
+#' @author Jon Clayden
+#' @seealso The DICOM standard, found online at \url{http://dicom.nema.org/}.
+#'   (Warning: may produce headaches!) Also \code{\link{readDicomDirectory}}
+#'   for information on how to create \code{\linkS4class{MriImage}} objects
+#'   from DICOM files.
+#' @references Please cite the following reference when using TractoR in your
+#' work:
+#' 
+#' J.D. Clayden, S. Muñoz Maniega, A.J. Storkey, M.D. King, M.E. Bastin & C.A.
+#' Clark (2011). TractoR: Magnetic resonance imaging and tractography with R.
+#' Journal of Statistical Software 44(8):1-18.
+#' \url{http://www.jstatsoft.org/v44/i08/}.
+#' @export
+readDicomFile <- function (fileName, checkFormat = TRUE, stopTag = NULL, ignoreTransferSyntax = FALSE, ascii = TRUE)
 {
     fileName <- expandFileName(fileName)
     
@@ -99,12 +163,9 @@ newDicomMetadataFromFile <- function (fileName, checkFormat = TRUE, dictionary =
     explicitTypes <- TRUE
     deferredTransferSyntax <- NULL
     tagOffset <- 0
-    dataOffset <- NULL
+    dataOffset <- dataLength <- NA
+    asciiFields <- NULL
     
-    if (is.null(dictionary))
-        data("dictionary", package="tractor.base", envir=environment(NULL))
-    dictionary$type <- as.vector(dictionary$type)
-    typeCol <- which(colnames(dictionary) == "type")
     connection <- file(fileName, "rb")
     on.exit(close(connection))
     
@@ -278,23 +339,20 @@ newDicomMetadataFromFile <- function (fileName, checkFormat = TRUE, dictionary =
             }
             
             if (!is.null(stopTag) && currentGroup == stopTag[1] && currentElement == stopTag[2])
-            {
-                dataOffset <- NA
-                dataLength <- NA
                 break
-            }
         }
         
-        if (is.null(dataOffset))
-            invisible (NULL)
-        else
+        if (duplicateTags)
+            flag(OL$Warning, "Duplicated DICOM tags detected - only the first value will be kept")
+        
+        if (ascii)
         {
-            if (duplicateTags)
-                flag(OL$Warning, "Duplicated DICOM tags detected - only the first value will be kept")
-            
-            tags <- data.frame(groups=groups, elements=elements, types=types, values=values, stringsAsFactors=FALSE)
-            invisible (DicomMetadata$new(source=fileName, tags=tags, tagOffset=as.integer(tagOffset), dataOffset=as.integer(dataOffset), dataLength=as.integer(dataLength), explicitTypes=explicitTypes, endian=endian))
+            asciiHeader <- ore.search(ore("### ASCCONV BEGIN[^#]+###(.+)### ASCCONV END ###",options="m"), ore.file(fileName,binary=TRUE))[,1]
+            asciiFields <- ore.split(ore("\n",syntax="fixed"), asciiHeader)
         }
+        
+        tags <- data.frame(groups=groups, elements=elements, types=types, values=values, stringsAsFactors=FALSE)
+        invisible (DicomMetadata$new(source=fileName, tags=tags, tagOffset=as.integer(tagOffset), dataOffset=as.integer(dataOffset), dataLength=as.integer(dataLength), explicitTypes=explicitTypes, endian=endian, asciiFields=as.character(asciiFields)))
     }
     else
         invisible (NULL)
